@@ -81,6 +81,8 @@
 --
 --   * Uninterpreted sorts, and proofs over such sorts, potentially with axioms.
 --
+--   *  Algebraic data types, including recursive fields..
+--
 --   * Ability to define SMTLib functions, generated directly from Haskell versions,
 --     including support for recursive and mutually recursive functions.
 --
@@ -156,7 +158,7 @@
 -- solvers are not good at proofs that require induction, or those that require complex chains of reasoning. Induction is necessary to reason about
 -- any recursive algorithm, and most such proofs require carefully constructed equational steps.
 --
--- SBV allows for a style of semi-automated theorem proving, called TP. which can be used to construct such proofs.
+-- SBV allows for a style of semi-automated theorem proving, called TP, which can be used to construct such proofs.
 -- The documentation includes example proofs for many list functions, and even inductive proofs for
 -- the familiar insertion, merge, quick-sort algorithms, along with a proof that the square-root of 2 is irrational.
 -- While a proper theorem prover (such as Lean, Isabelle etc.) is a more appropriate choice for such proofs, with some
@@ -211,7 +213,7 @@ module Data.SBV (
   , SWord8, SWord16, SWord32, SWord64, SWord, WordN
   -- *** Signed bit-vectors
   , SInt8, SInt16, SInt32, SInt64, SInt, IntN
-  -- *** Converting between fixed-size and arbitrary bitvectors
+  -- *** Converting between fixed-size and arbitrary bit-vectors
   , BVIsNonZero, FromSized, ToSized, fromSized, toSized
   -- ** Unbounded integers
   -- $unboundedLimitations
@@ -239,11 +241,11 @@ module Data.SBV (
   , SList
   -- ** Symbolic enumerators
   , EnumSymbolic(..), sEnum
+  -- ** Symbolic case-expressions
+  , sCase
   -- ** Tuples
   -- $tuples
   , SymTuple, STuple, STuple2, STuple3, STuple4, STuple5, STuple6, STuple7, STuple8
-  -- ** Sum types
-  , SMaybe, SEither
   -- ** Sets
   , RCSet(..), SSet
   -- * Arrays of symbolic values
@@ -270,10 +272,14 @@ module Data.SBV (
   , sString, sString_
   , sList, sList_
   , sTuple, sTuple_
-  , sEither, sEither_
-  , sMaybe, sMaybe_
   , sSet, sSet_
 
+  -- * Symbolic 'Maybe'
+  , SMaybe, sMaybe, sMaybe_, sMaybes
+
+  -- * Symbolic 'Either'
+  , SEither, sEither, sEither_, sEithers
+
   -- ** List of values
   -- $createSyms
   , sBools
@@ -294,8 +300,6 @@ module Data.SBV (
   , sStrings
   , sLists
   , sTuples
-  , sEithers
-  , sMaybes
   , sSets
 
   -- * Symbolic Equality and Comparisons
@@ -343,13 +347,9 @@ module Data.SBV (
   -- ** Showing values in detail
   , crack
 
-  -- * Enumerations
-  -- $enumerations
-  , mkSymbolicEnumeration
-
-  -- * Uninterpreted sorts, constants, and functions
-  -- $uninterpreted
-  , mkUninterpretedSort
+  -- * Symbolic data types
+  -- $symbolicADT
+  , mkSymbolic
 
   -- * Stopping unrolling: Defined functions
   , SMTDefinable(..), smtHOFunction, Closure(..), registerType
@@ -442,7 +442,7 @@ module Data.SBV (
   -- ** Programmable model extraction
   -- $programmableExtraction
   , Modelable(..), displayModels, extractModels
-  , getModelDictionaries, getModelValues, getModelUninterpretedValues
+  , getModelDictionaries, getModelValues
 
   -- * SMT Interface
   , SMTConfig(..), TPOptions(..), Timing(..), SMTLibVersion(..), Solver(..), SMTSolver(..)
@@ -497,7 +497,7 @@ import Data.SBV.Core.Model      hiding (assertWithPenalty, minimize, maximize,
                                         sRational, sRational_, sRationals,
                                         sWord8, sWord8_, sWord8s, sWord16, sWord16_, sWord16s,
                                         sWord32, sWord32_, sWord32s, sWord64, sWord64_, sWord64s,
-                                        sMaybe, sMaybe_, sMaybes, sEither, sEither_, sEithers, sSet, sSet_, sSets,
+                                        sSet, sSet_, sSets,
                                         sArray, sArray_, sArrays,
                                         sBarrelRotateLeft, sBarrelRotateRight, zeroExtend, signExtend, sObserve)
 
@@ -508,6 +508,9 @@ import Data.SBV.Core.Kind
 
 import Data.SBV.Core.SizedFloats
 
+import Data.SBV.Maybe hiding (map)
+import Data.SBV.Either
+
 import Data.SBV.Core.Floating
 import Data.SBV.Core.Symbolic   ( MonadSymbolic(..), SymbolicT, registerKind
                                 , ProgInfo(..), rProgInfo, SpecialRelOp(..), UICodeKind(UINone)
@@ -548,6 +551,7 @@ import GHC.TypeLits (KnownNat, type (<=), type (+), type (-))
 import Data.Char (isSpace, isPunctuation)
 import Data.SBV.List (EnumSymbolic(..))
 import Data.SBV.SEnum (sEnum)
+import Data.SBV.SCase (sCase)
 
 import Data.SBV.Rational
 
@@ -1114,78 +1118,72 @@ where the backend solver can be queried to obtain an unsat core in case the cons
 See 'Data.SBV.Control.getUnsatCore' for details and "Documentation.SBV.Examples.Queries.UnsatCore" for an example use case.
 -}
 
-{- $uninterpreted
+{- $symbolicADT
 Users can introduce new uninterpreted sorts simply by defining an empty data-type in Haskell and registering it as such. The
 following example demonstrates:
 
   @
      data B
-     mkUninterpretedSort ''B
+     mkSymbolic [''B]
   @
 
-(Note that you'll also need to use pragmas @TemplateHaskell@, @StandAloneDeriving@, @DeriveDataTypeable@, and @DeriveAnyClass@ for this to work, follow GHC's error messages!)
-
 This is all it takes to introduce @B@ as an uninterpreted sort in SBV, which makes the type @SBV B@ automagically become available as the type
 of symbolic values that ranges over @B@ values. Note that this will also introduce the type @SB@ into your environment, which is a synonym
 for @SBV B@.
 
-
-Uninterpreted functions over both uninterpreted and regular sorts can be declared using the facilities introduced by
-the 'Data.SBV.Core.Model.SMTDefinable' class.
--}
-
-{- $enumerations
 If the uninterpreted sort definition takes the form of an enumeration (i.e., a simple data type with all nullary constructors), then
-you can use the 'mkSymbolicEnumeration' function to turn it into an enumeration in SMTLib.
-A simple example is:
+it will turn into an enumeration in SMTLib.  A simple example is:
 
 @
-    data X = A | B | C deriving (Enum, Bounded)
-    mkSymbolicEnumeration ''X
+    data X = A | B | C
+    mkSymbolic [''X]
 @
 
-Note the magic incantation @mkSymbolicEnumeration ''X@. For this to work, you need to have the following
-options turned on:
+Note the magic incantation @mkSymbolic [''X]@, requires certain extensions to be turned on. Simply follow GHC's advice.
 
->   LANGUAGE TemplateHaskell
->   LANGUAGE StandaloneDeriving
->   LANGUAGE DeriveDataTypeable
->   LANGUAGE DeriveAnyClass
->   LANGUAGE FlexibleInstances
-
-and your own declaration must have instances of 'Enum' and 'Bounded'. (The instances can be derived, as above.)
-This will automatically introduce the type:
+SBV also supports good old ADT's as well, with fields. The support for this is similar, where SBV will create the
+corresponding datatype in a symbolic manner:
 
 @
-    type SX = SBV X
+-- | A basic arithmetic expression type.
+data Expr = Num Integer
+          | Var String
+          | Add Expr Expr
+          | Mul Expr Expr
+          | Let String Expr Expr
+
+-- | Create a symbolic version of expressions.
+mkSymbolic [''Expr]
 @
 
-along with symbolic values of each of the enumerated values @sA@, @sB@, and @sC@. This way,
-you can refer to the symbolic version as @SX@, treating it as a regular symbolic type ranging over the values @A@, @B@, and @C@. Such values can be compared for equality, and with the usual
-other comparison operators, such as @.==@, @./=@, @.>@, @.>=@, @<@, and @<=@. For each enumerated value @X@, the symbolic versions @sX@ is defined to be equal to @literal X@. Furthermore, the symbolic type will be an instance of 'EnumSymbolic', allowing you to use
-arithmetic progressions on symbolic values.
+These types can also be parameterized, per usual Haskell usage.
 
-A simple query would look like:
+We also support a symbolic case-expression quasi-quoter, allowing us to write:
 
 @
-     allSat $ \x -> x .== (x :: SX)
+eval :: SExpr -> SInteger
+eval = go SL.nil
+ where go :: SList (String, Integer) -> SExpr -> SInteger
+       go = smtFunction "eval" $ \env expr -> [sCase|Expr expr of
+                                                 Num i     -> i
+                                                 Var s     -> get env s
+                                                 Add l r   -> go env l + go env r
+                                                 Mul l r   -> go env l * go env r
+                                                 Let s e r -> go (tuple (s, go env e) SL..: env) r
+                                              |]
+
+       get :: SList (String, Integer) -> SString -> SInteger
+       get = smtFunction "get" $ \env s -> ite (SL.null env) 0
+                                         $ let (k, v) = untuple (SL.head env)
+                                           in ite (s .== k) v (get (SL.tail env) s)
 @
 
-which would list all three elements of this domain as satisfying solutions.
+which defines an interpreter for this data-type. Such definitions also come with an induction principle
+to perform TP based proofs on. These can be accessed using the 'Data.SBV.TP.inductiveLemma' function.
 
-@
-     Solution #1:
-       s0 = A :: X
-     Solution #2:
-       s0 = B :: X
-     Solution #3:
-       s0 = C :: X
-     Found 3 different solutions.
-@
-
-Note that the result is properly typed as @X@ elements; these are not mere strings.
-
-See "Documentation.SBV.Examples.Misc.Enumerate" for an extended example on how to use symbolic enumerations.
+The argument to @mkSymbolic@ is typically a list of types. The requirement is that if the types you pass on
+are mutually recursively defined, you should give them as a list. Otherwise, you can give them all together,
+or one at a time.
 -}
 
 {- $cardIntro
@@ -1400,7 +1398,7 @@ In this case the numerator is off by 148.
 -}
 
 -- | An implementation of rotate-left, using a barrel shifter like design. Only works when both
--- arguments are finite bitvectors, and furthermore when the second argument is unsigned.
+-- arguments are finite bit-vectors, and furthermore when the second argument is unsigned.
 -- The first condition is enforced by the type, but the second is dynamically checked.
 -- We provide this implementation as an alternative to `sRotateLeft` since SMTLib logic
 -- does not support variable argument rotates (as opposed to shifts), and thus this
@@ -1435,7 +1433,7 @@ bvExtract :: forall i j n bv proxy. ( KnownNat n, BVIsNonZero n, SymVal (bv n)
                                       -> SBV (bv (i - j + 1))   -- ^ Output is of size @i - j + 1@
 bvExtract = CD.bvExtract
 
--- | Join two bitvectors.
+-- | Join two bit-vectors.
 --
 -- >>> prove $ \x y -> x .== bvExtract (Proxy @79) (Proxy @71) ((x :: SWord 9) # (y :: SWord 71))
 -- Q.E.D.
@@ -1756,7 +1754,11 @@ checkSpecialRelation op rel = SBV $ SVal KBool $ Right $ cache result
                        unless (op `elem` curSpecialRels) $ do
 
                           registerKind st ka
-                          nm' <- newUninterpreted st (UIGiven nm) Nothing (SBVType [ka, ka, KBool]) (UINone True)
+                          uop <- newUninterpreted st (UIGiven nm) Nothing (SBVType [ka, ka, KBool]) (UINone True)
+
+                          let nm' = case uop of
+                                      Uninterpreted s -> s
+                                      _               -> error "Data.SBV: Impossible happened: checkSpecialRelation received: " ++ show op
 
                           -- Add to the end so if we get incremental ones the order doesn't change for old ones!
                           modifyIORef' (rProgInfo st) (\u -> u{progSpecialRels = curSpecialRels ++ [iop]})
