@@ -10,11 +10,10 @@
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveGeneric       #-}
 {-# LANGUAGE DeriveDataTypeable  #-}
+{-# LANGUAGE DeriveGeneric       #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeApplications    #-}
-{-# LANGUAGE Rank2Types          #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -100,21 +99,19 @@ instance (HasKind a, HasKind b) => HasKind (ArrayModel a b) where
 -- | A constant value.
 -- Note: If you add a new constructor here, make sure you add the
 -- corresponding equality in the instance "Eq CVal" and "Ord CVal"!
-data CVal = CAlgReal  !AlgReal                -- ^ Algebraic real
-          | CInteger  !Integer                -- ^ Bit-vector/unbounded integer
-          | CFloat    !Float                  -- ^ Float
-          | CDouble   !Double                 -- ^ Double
-          | CFP       !FP                     -- ^ Arbitrary float
-          | CRational Rational                -- ^ Rational
-          | CChar     !Char                   -- ^ Character
-          | CString   !String                 -- ^ String
-          | CList     ![CVal]                 -- ^ List
-          | CSet      !(RCSet CVal)           -- ^ Set. Can be regular or complemented.
-          | CUserSort !(Maybe Int, String)    -- ^ Value of an uninterpreted/user kind. The Maybe Int shows index position for enumerations
-          | CTuple    ![CVal]                 -- ^ Tuple
-          | CMaybe    !(Maybe CVal)           -- ^ Maybe
-          | CEither   !(Either CVal CVal)     -- ^ Disjoint union
-          | CArray    !(ArrayModel CVal CVal) -- ^ Arrays are backed by look-up tables concretely
+data CVal = CAlgReal  !AlgReal                 -- ^ Algebraic real
+          | CInteger  !Integer                 -- ^ Bit-vector/unbounded integer
+          | CFloat    !Float                   -- ^ Float
+          | CDouble   !Double                  -- ^ Double
+          | CFP       !FP                      -- ^ Arbitrary float
+          | CRational Rational                 -- ^ Rational
+          | CChar     !Char                    -- ^ Character
+          | CString   !String                  -- ^ String
+          | CList     ![CVal]                  -- ^ List
+          | CSet      !(RCSet CVal)            -- ^ Set. Can be regular or complemented.
+          | CADT      (String, [(Kind, CVal)]) -- ^ ADT: Constructor, and fields
+          | CTuple    ![CVal]                  -- ^ Tuple
+          | CArray    !(ArrayModel CVal CVal)  -- ^ Arrays are backed by look-up tables concretely
           deriving (G.Data, Generic, NFData)
 
 -- | Assign a rank to constant values, this is structural and helps with ordering
@@ -129,11 +126,9 @@ cvRank CChar     {} =  6
 cvRank CString   {} =  7
 cvRank CList     {} =  8
 cvRank CSet      {} =  9
-cvRank CUserSort {} = 10
+cvRank CADT      {} = 10
 cvRank CTuple    {} = 11
-cvRank CMaybe    {} = 12
-cvRank CEither   {} = 13
-cvRank CArray    {} = 14
+cvRank CArray    {} = 12
 
 -- | Eq instance for CVal. Note that we cannot simply derive Eq/Ord, since CVAlgReal doesn't have proper
 -- instances for these when values are infinitely precise reals. However, we do
@@ -149,10 +144,8 @@ instance Eq CVal where
   CString   a == CString   b = a == b
   CList     a == CList     b = a == b
   CSet      a == CSet      b = a `eqRCSet` b
-  CUserSort a == CUserSort b = a == b
   CTuple    a == CTuple    b = a == b
-  CMaybe    a == CMaybe    b = a == b
-  CEither   a == CEither   b = a == b
+  CADT      a == CADT      b = a == b
 
   -- This is legit since we don't use this equality for actual semantic" equality, but rather as an index into maps
   CArray    (ArrayModel a1 d1) == CArray (ArrayModel a2 d2) = (a1, d1) == (a2, d2)
@@ -179,10 +172,8 @@ instance Ord CVal where
   CString   a `compare` CString   b = a `compare`                  b
   CList     a `compare` CList     b = a `compare`                  b
   CSet      a `compare` CSet      b = a `compareRCSet`             b
-  CUserSort a `compare` CUserSort b = a `compare`                  b
   CTuple    a `compare` CTuple    b = a `compare`                  b
-  CMaybe    a `compare` CMaybe    b = a `compare`                  b
-  CEither   a `compare` CEither   b = a `compare`                  b
+  CADT      a `compare` CADT      b = a `compare`                  b
 
   -- This is legit since we don't use this equality for actual semantic order, but rather as an index into maps
   CArray    (ArrayModel a1 d1) `compare` CArray (ArrayModel a2 d2) = (a1, d1) `compare` (a2, d2)
@@ -201,8 +192,8 @@ instance Ord CVal where
 
 -- | A t'CV' represents a concrete word of a fixed size:
 -- For signed words, the most significant digit is considered to be the sign.
-data CV = CV { _cvKind  :: !Kind
-             , cvVal    :: !CVal
+data CV = CV { cvKind  :: !Kind
+             , cvVal   :: !CVal
              }
              deriving (Eq, Ord, G.Data, NFData, Generic)
 
@@ -332,12 +323,10 @@ mapCV r i f d af ra x  = normCV $ CV (kindOf x) $ case cvVal x of
                                                     CRational a -> CRational (ra a)
                                                     CChar{}     -> error "Data.SBV.mapCV: Unexpected call through mapCV with chars!"
                                                     CString{}   -> error "Data.SBV.mapCV: Unexpected call through mapCV with strings!"
-                                                    CUserSort{} -> error "Data.SBV.mapCV: Unexpected call through mapCV with uninterpreted sorts!"
+                                                    CADT{}      -> error "Data.SBV.mapCV: Unexpected call through mapCV with ADTs!"
                                                     CList{}     -> error "Data.SBV.mapCV: Unexpected call through mapCV with lists!"
                                                     CSet{}      -> error "Data.SBV.mapCV: Unexpected call through mapCV with sets!"
                                                     CTuple{}    -> error "Data.SBV.mapCV: Unexpected call through mapCV with tuples!"
-                                                    CMaybe{}    -> error "Data.SBV.mapCV: Unexpected call through mapCV with maybe!"
-                                                    CEither{}   -> error "Data.SBV.mapCV: Unexpected call through mapCV with either!"
                                                     CArray{}    -> error "Data.SBV.mapCV: Unexpected call through mapCV with arrays!"
 
 -- | Map a binary function through a t'CV'.
@@ -357,11 +346,8 @@ mapCV2 r i f d af ra x y = case (cvSameType x y, cvVal x, cvVal y) of
                             (True, CRational a, CRational b) -> normCV $ CV (kindOf x) (CRational (ra a b))
                             (True, CChar{},     CChar{})     -> unexpected "chars!"
                             (True, CString{},   CString{})   -> unexpected "strings!"
-                            (True, CUserSort{}, CUserSort{}) -> unexpected "uninterpreted constants!"
                             (True, CList{},     CList{})     -> unexpected "lists!"
                             (True, CTuple{},    CTuple{})    -> unexpected "tuples!"
-                            (True, CMaybe{},    CMaybe{})    -> unexpected "maybes!"
-                            (True, CEither{},   CEither{})   -> unexpected "eithers!"
                             _                                -> unexpected $ "incompatible args: " ++ show (x, y)
    where unexpected w = error $ unlines [ ""
                                         , "*** Data.SBV.mapCV2: Unexpected call through mapCV2 with " ++ w
@@ -386,21 +372,19 @@ showCV shk w = sh (cvVal w) ++ kInfo
 
         wk = kindOf w
 
-        sh (CAlgReal  v) = show v
-        sh (CInteger  v) = show v
-        sh (CFloat    v) = show v
-        sh (CDouble   v) = show v
-        sh (CFP       v) = show v
-        sh (CRational v) = show v
-        sh (CChar     v) = show v
-        sh (CString   v) = show v
-        sh (CUserSort v) = snd  v
-        sh (CList     v) = shL  v
-        sh (CSet      v) = shS  v
-        sh (CTuple    v) = shT  v
-        sh (CMaybe    v) = shM  v
-        sh (CEither   v) = shE  v
-        sh (CArray    v) = shA  v
+        sh (CAlgReal  v) = show  v
+        sh (CInteger  v) = show  v
+        sh (CFloat    v) = show  v
+        sh (CDouble   v) = show  v
+        sh (CFP       v) = show  v
+        sh (CRational v) = show  v
+        sh (CChar     v) = show  v
+        sh (CString   v) = show  v
+        sh (CADT      c) = shADT c
+        sh (CList     v) = shL   v
+        sh (CSet      v) = shS   v
+        sh (CTuple    v) = shT   v
+        sh (CArray    v) = shA   v
 
         shL xs = "[" ++ intercalate "," (map (showCV False . CV ke) xs) ++ "]"
           where ke = case wk of
@@ -424,34 +408,24 @@ showCV shk w = sh (cvVal w) ++ kInfo
                         KTuple ks | length ks == length xs -> zipWith (\k x -> showCV False (CV k x)) ks xs
                         _   -> error $ "Data.SBV.showCV: Impossible happened, expected tuple (of length " ++ show (length xs) ++ "), got: " ++ show wk
 
-        shM :: Maybe CVal -> String
-        shM c = case (c, wk) of
-                  (Nothing, KMaybe{}) -> "Nothing"
-                  (Just x,  KMaybe k) -> "Just " ++ paren (showCV False (CV k x))
-                  _                   -> error $ "Data.SBV.showCV: Impossible happened, expected maybe, got: " ++ show wk
-
-        shE :: Either CVal CVal -> String
-        shE val
-          | KEither k1 k2 <- wk = case val of
-                                    Left  x -> "Left "  ++ paren (showCV False (CV k1 x))
-                                    Right y -> "Right " ++ paren (showCV False (CV k2 y))
-          | True                = error $ "Data.SBV.showCV: Impossible happened, expected sum, got: " ++ show wk
-
         shA :: ArrayModel CVal CVal -> String
         shA (ArrayModel assocs def)
           | KArray k1 k2 <- wk = "([" ++ intercalate "," [showCV False (CV (KTuple [k1, k2]) (CTuple [a, b])) | (a, b) <- assocs] ++ "], " ++ showCV False (CV k2 def) ++ ")"
           | True               = error $ "Data.SBV.showCV: Impossible happened, expected array, got: " ++ show wk
 
-        -- kind of crude, but works ok
-        paren v
-          | needsParen = '(' : v ++ ")"
-          | True       = v
-          where needsParen = case dropWhile isSpace v of
-                               []         -> False
-                               rest@(x:_) -> x == '-' || (any isSpace rest && x `notElem` "{[(")
+        shADT (c, kvs)
+          | null @[] flds = c
+          | True          = unwords (c : map wrap flds)
+          where wrap v
+                 | take 1 v `elem` ["(", "[", "{"]  = v
+                 | any isSpace v || take 1 v == "-" = '(' : v ++ ")"
+                 | True                             = v
+
+                flds = map (\(k, v) -> showCV False (CV k v)) kvs
 
 -- | Create a constant word from an integral.
 mkConstCV :: Integral a => Kind -> a -> CV
+mkConstCV k@(KVar{})      _ = error $ "mkConstCV: Unexpected kind: " ++ show k
 mkConstCV KBool           a = normCV $ CV KBool      (CInteger  (toInteger a))
 mkConstCV k@KBounded{}    a = normCV $ CV k          (CInteger  (toInteger a))
 mkConstCV KUnbounded      a = normCV $ CV KUnbounded (CInteger  (toInteger a))
@@ -462,18 +436,18 @@ mkConstCV k@(KFP eb sb)   a = normCV $ CV k          (CFP       (fpFromInteger e
 mkConstCV KRational       a = normCV $ CV KRational  (CRational (fromInteger (toInteger a)))
 mkConstCV KChar           a = error $ "Unexpected call to mkConstCV (Char) with value: "   ++ show (toInteger a)
 mkConstCV KString         a = error $ "Unexpected call to mkConstCV (String) with value: " ++ show (toInteger a)
-mkConstCV (KUserSort s _) a = error $ "Unexpected call to mkConstCV with user kind: " ++ s ++ " with value: " ++ show (toInteger a)
+mkConstCV (KApp s _)      a = error $ "Unexpected call to mkConstCV with kind: " ++ s ++ " with value: " ++ show (toInteger a)
+mkConstCV (KADT s _ _)    a = error $ "Unexpected call to mkConstCV with ADT: "  ++ s ++ " with value: " ++ show (toInteger a)
 mkConstCV k@KList{}       a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
 mkConstCV k@KSet{}        a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
 mkConstCV k@KTuple{}      a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
-mkConstCV k@KMaybe{}      a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
-mkConstCV k@KEither{}     a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
 mkConstCV k@KArray{}      a = error $ "Unexpected call to mkConstCV (" ++ show k ++ ") with value: " ++ show (toInteger a)
 
 -- | Generate a random constant value ('CVal') of the correct kind. We error out for a completely uninterpreted type.
 randomCVal :: Kind -> IO CVal
 randomCVal k =
   case k of
+    KVar{}             -> error $ "randomCVal: Unexpected kind: " ++ show k
     KBool              -> CInteger  <$> randomRIO (0, 1)
     KBounded s w       -> CInteger  <$> randomRIO (bounds s w)
     KUnbounded         -> CInteger  <$> randomIO
@@ -494,10 +468,14 @@ randomCVal k =
                              CString <$> replicateM l (chr <$> randomRIO (0, 255))
     KChar              -> CChar . chr <$> randomRIO (0, 255)
 
-    KUserSort s es     -> case es of
-                            Just vs@(_:_) -> do i <- randomRIO (0, length vs - 1)
-                                                pure $ CUserSort (Just i, vs !! i)
-                            _             -> error $ "randomCVal: Not supported for completely uninterpreted type: " ++ s
+    -- TODO: Can we do something here?
+    KApp s _           -> error $ "randomCVal: Not supported for KApp: " ++ s
+
+    KADT _ _ cstrs@(_:_) -> do i <- randomRIO (0, length cstrs - 1)
+                               let (c, fks) = cstrs !! i
+                               vs <- mapM randomCVal fks
+                               pure $ CADT (c, zip fks vs)
+    KADT s _ _         -> error $ "randomCVal: Not supported for ADT:  " ++ s
 
     KList ek           -> do l <- randomRIO (0, 100)
                              CList <$> replicateM l (randomCVal ek)
@@ -509,16 +487,6 @@ randomCVal k =
 
     KTuple ks          -> CTuple <$> traverse randomCVal ks
 
-    KMaybe ke          -> do i <- randomIO
-                             if i
-                                then return $ CMaybe Nothing
-                                else CMaybe . Just <$> randomCVal ke
-
-    KEither k1 k2      -> do i <- randomIO
-                             if i
-                                then CEither . Left  <$> randomCVal k1
-                                else CEither . Right <$> randomCVal k2
-
     KArray k1 k2       -> do l   <- randomRIO (0, 100)
                              ks  <- replicateM l (randomCVal k1)
                              vs  <- replicateM l (randomCVal k2)
