@@ -17,11 +17,10 @@
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE FlexibleInstances     #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE PatternGuards         #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE TypeApplications      #-}
-{-# LANGUAGE TypeOperators         #-}
 {-# LANGUAGE TypeFamilies          #-}
+{-# LANGUAGE TypeOperators         #-}
 {-# LANGUAGE UndecidableInstances  #-}
 
 {-# OPTIONS_GHC -Wall -Werror -Wno-orphans #-}
@@ -33,12 +32,10 @@ module Data.SBV.Core.Data
  , SWord, SInt, WordN, IntN
  , SRational
  , SChar, SString, SList
- , SEither, SMaybe, SArray, ArrayModel(..)
+ , SArray, ArrayModel(..)
  , STuple, STuple2, STuple3, STuple4, STuple5, STuple6, STuple7, STuple8
  , RCSet(..), SSet
  , nan, infinity, sNaN, sInfinity, RoundingMode(..), SRoundingMode
- , sRoundNearestTiesToEven, sRoundNearestTiesToAway, sRoundTowardPositive, sRoundTowardNegative, sRoundTowardZero
- , sRNE, sRNA, sRTP, sRTN, sRTZ
  , SymVal(..)
  , CV(..), CVal(..), AlgReal(..), AlgRealPoly(..), ExtCV(..), GeneralizedCV(..), isRegularCV, cvSameType, cvToBool
  , mkConstCV , mapCV, mapCV2
@@ -50,7 +47,7 @@ module Data.SBV.Core.Data
  , SBVExpr(..), newExpr
  , cache, Cached, uncache, HasKind(..)
  , Op(..), PBOp(..), FPOp(..), StrOp(..), RegExOp(..), SeqOp(..), RegExp(..), NamedSymVar(..), OvOp(..), getTableIndex
- , SBVPgm(..), Symbolic, runSymbolic, State, getPathCondition, extendPathCondition
+ , SBVPgm(..), Symbolic, runSymbolic, State, SInfo(..), getSInfo, getPathCondition, extendPathCondition
  , inSMTMode, SBVRunMode(..), Kind(..), Outputtable(..), Result(..)
  , SolverContext(..), internalConstraint, isCodeGenMode
  , SBVType(..), newUninterpreted
@@ -73,15 +70,16 @@ import Control.Monad          (void, replicateM)
 import Control.Monad.Trans    (liftIO, MonadIO)
 import Data.Int               (Int8, Int16, Int32, Int64)
 import Data.Word              (Word8, Word16, Word32, Word64)
-import Data.List              (elemIndex)
 
 import Data.Kind (Type)
 import Data.Proxy
 import Data.Typeable          (Typeable)
 
+import Data.IORef
+import qualified Data.Set as Set (toList)
+
 import GHC.Generics (Generic, U1(..), M1(..), (:*:)(..), K1(..), (:+:)(..))
 import qualified GHC.Generics  as G
-import qualified Data.Generics as G (Data(..))
 
 import System.Random
 
@@ -96,6 +94,7 @@ import Data.SBV.Core.Operations
 import Data.SBV.Control.Types
 
 import Data.SBV.Utils.Lib
+import Data.SBV.Utils.Numeric (RoundingMode(..))
 
 import Test.QuickCheck (Arbitrary(..))
 
@@ -196,12 +195,6 @@ type SRational = SBV Rational
 -- Note that lists can be nested, i.e., we do allow lists of lists of ... items.
 type SList a = SBV [a]
 
--- | Symbolic 'Either'
-type SEither a b = SBV (Either a b)
-
--- | Symbolic 'Maybe'
-type SMaybe a = SBV (Maybe a)
-
 -- | Symbolic arrays. A symbolic array is more akin to a function in SMTLib (and thus in SBV),
 -- as opposed to contagious-storage with a finite range as found in many programming languages.
 -- Additionally, the domain uses object-equality in the SMTLib semantics. Object equality is
@@ -336,52 +329,9 @@ sAny f = sOr  . map f
 sAll :: (a -> SBool) -> [a] -> SBool
 sAll f = sAnd . map f
 
--- | 'RoundingMode' can be used symbolically
-instance SymVal RoundingMode
-
 -- | The symbolic variant of 'RoundingMode'
 type SRoundingMode = SBV RoundingMode
 
--- | Symbolic variant of 'RoundNearestTiesToEven'
-sRoundNearestTiesToEven :: SRoundingMode
-sRoundNearestTiesToEven = literal RoundNearestTiesToEven
-
--- | Symbolic variant of 'RoundNearestTiesToAway'
-sRoundNearestTiesToAway :: SRoundingMode
-sRoundNearestTiesToAway = literal RoundNearestTiesToAway
-
--- | Symbolic variant of 'RoundTowardPositive'
-sRoundTowardPositive :: SRoundingMode
-sRoundTowardPositive = literal RoundTowardPositive
-
--- | Symbolic variant of 'RoundTowardNegative'
-sRoundTowardNegative :: SRoundingMode
-sRoundTowardNegative = literal RoundTowardNegative
-
--- | Symbolic variant of 'RoundTowardZero'
-sRoundTowardZero :: SRoundingMode
-sRoundTowardZero = literal RoundTowardZero
-
--- | Alias for 'sRoundNearestTiesToEven'
-sRNE :: SRoundingMode
-sRNE = sRoundNearestTiesToEven
-
--- | Alias for 'sRoundNearestTiesToAway'
-sRNA :: SRoundingMode
-sRNA = sRoundNearestTiesToAway
-
--- | Alias for 'sRoundTowardPositive'
-sRTP :: SRoundingMode
-sRTP = sRoundTowardPositive
-
--- | Alias for 'sRoundTowardNegative'
-sRTN :: SRoundingMode
-sRTN = sRoundTowardNegative
-
--- | Alias for 'sRoundTowardZero'
-sRTZ :: SRoundingMode
-sRTZ = sRoundTowardZero
-
 -- | A 'Show' instance is not particularly "desirable," when the value is symbolic,
 -- but we do need this instance as otherwise we cannot simply evaluate Haskell functions
 -- that return symbolic values and have their constant values printed easily!
@@ -436,7 +386,7 @@ newtype Forall (nm :: Symbol) a = Forall (SBV a)
 -- will be prefixed in front of @_1@, @_2@, ..., @_n@ to form the names of the variables.
 newtype ExistsN (n :: Nat) (nm :: Symbol) a = ExistsN [SBV a]
 
--- | Exactly @n@ universal symbolic variables, used in in building quantified constraints. The name attached
+-- | Exactly @n@ universal symbolic variables, used in building quantified constraints. The name attached
 -- will be prefixed in front of @_1@, @_2@, ..., @_n@ to form the names of the variables.
 newtype ForallN (n :: Nat) (nm :: Symbol) a = ForallN [SBV a]
 
@@ -557,6 +507,14 @@ registerType :: forall a m. (MonadIO m, SolverContext m, HasKind a) => Proxy a -
 registerType _ = do st <- contextState
                     liftIO $ registerKind st (kindOf (Proxy @a))
 
+-- | Various info we use in recoverKinded value
+newtype SInfo = SInfo { sInfoKinds :: [Kind] }
+
+-- | Turn state into SInfo
+getSInfo :: MonadIO m => State -> m SInfo
+getSInfo st = do rk <- liftIO $ readIORef (rUsedKinds st)
+                 pure $ SInfo { sInfoKinds = Set.toList rk }
+
 -- | A class representing what can be returned from a symbolic computation.
 class Outputtable a where
   -- | Generalization of 'Data.SBV.output'
@@ -602,6 +560,10 @@ class (HasKind a, Typeable a, Arbitrary a) => SymVal a where
   -- | Generalization of 'Data.SBV.mkSymVal'
   mkSymVal :: MonadSymbolic m => VarContext -> Maybe String -> m (SBV a)
 
+  -- | Certain types (ADTs) might need to do further initialization.
+  mkSymValInit :: State -> SBV a -> IO ()
+  mkSymValInit _ _ = pure ()
+
   -- | Turn a literal constant to symbolic
   literal :: a -> SBV a
 
@@ -615,32 +577,13 @@ class (HasKind a, Typeable a, Arbitrary a) => SymVal a where
   -- If the underlying type is bounded, we have a default below. Otherwise it's nothing.
   minMaxBound :: Maybe (a, a)
 
-  -- minimal complete definition: Nothing.
-  -- Giving no instances is okay when defining an uninterpreted/enumerated sort, but otherwise you really
-  -- want to define: literal, fromCV, mkSymVal
+  {-# MINIMAL literal, fromCV #-}
 
-  default mkSymVal :: (MonadSymbolic m, Read a, G.Data a) => VarContext -> Maybe String -> m (SBV a)
-  mkSymVal vc mbNm = SBV <$> (symbolicEnv >>= liftIO . svMkSymVar vc k mbNm)
-    where -- NB.A call of the form
-          --      constructUKind (Proxy @a)
-          -- would be wrong here, as it would uninterpret the Proxy datatype!
-          -- So, we have to use the dreaded undefined value in this case.
-          k = constructUKind (undefined :: a)
-
-  default literal :: Show a => a -> SBV a
-  literal x = let k  = kindOf x
-                  sx = show x
-                  conts = case k of
-                           KUserSort _ cts -> cts
-                           _               -> Nothing
-                  mbIdx = case conts of
-                            Just xs -> sx `elemIndex` xs
-                            Nothing -> Nothing
-              in SBV $ SVal k (Left (CV k (CUserSort (mbIdx, sx))))
-
-  default fromCV :: Read a => CV -> a
-  fromCV (CV _ (CUserSort (_, s))) = read s
-  fromCV cv                        = error $ "Cannot convert CV " ++ show cv ++ " to kind " ++ show (kindOf (Proxy @a))
+  default mkSymVal :: MonadSymbolic m => VarContext -> Maybe String -> m (SBV a)
+  mkSymVal vc mbNm = do st <- symbolicEnv
+                        liftIO $ do v <- SBV <$> svMkSymVar vc (kindOf (undefined :: a)) mbNm st
+                                    mkSymValInit st v
+                                    pure v
 
   default minMaxBound :: Bounded a => Maybe (a, a)
   minMaxBound = Just (minBound, maxBound)
@@ -674,6 +617,11 @@ class (HasKind a, Typeable a, Arbitrary a) => SymVal a where
   unliteral (SBV (SVal _ (Left c))) = Just $ fromCV c
   unliteral _                       = Nothing
 
+  -- | Get the underlying CV, if available
+  unlitCV :: SBV a -> Maybe (Kind, CVal)
+  unlitCV (SBV (SVal _ (Left (CV k v)))) = Just (k, v)
+  unlitCV _                              = Nothing
+
   -- | Is the symbolic word concrete?
   isConcrete :: SBV a -> Bool
   isConcrete (SBV (SVal _ (Left _))) = True
@@ -859,7 +807,7 @@ bvExtract start end = SBV . svExtract i j . unSBV
    where i  = fromIntegral (natVal start)
          j  = fromIntegral (natVal end)
 
--- | Join two bitvectors.
+-- | Join two bit-vectors.
 (#) :: ( KnownNat n, BVIsNonZero n, SymVal (bv n)
        , KnownNat m, BVIsNonZero m, SymVal (bv m)
        ) => SBV (bv n)                     -- ^ First input, of size @n@, becomes the left side
@@ -1001,7 +949,7 @@ instance QNot r => QNot (ExistsN nm n a -> r) where
   type NegatesTo (ExistsN nm n a -> r) = ForallN nm n a -> NegatesTo r
   qNot f (ForallN xs) = qNot (f (ExistsN xs))
 
--- | Negate over an unique existential quantifier
+-- | Negate over a unique existential quantifier
 instance (QNot r, QuantifiedBool r, EqSymbolic (SBV a)) => QNot (ExistsUnique nm a -> r) where
   type NegatesTo (ExistsUnique nm a -> r) =  Forall nm a
                                           -> Exists (AppendSymbol nm "_eu1") a
