@@ -51,7 +51,9 @@ module Data.SBV.Core.Operations
 
 import Prelude hiding (Foldable(..))
 import Data.Bits (Bits(..))
-import Data.List (genericIndex, genericLength, genericTake, foldr, length, foldl', elem, nub, sort)
+import Data.List (genericIndex, genericLength, genericTake, foldr, length, foldl', elem, nub, sort, null, elemIndex)
+
+import Data.Maybe (isNothing)
 
 import Data.SBV.Core.AlgReals
 import Data.SBV.Core.Kind
@@ -492,29 +494,6 @@ cCompare k op x y =
       (CAlgReal     a, CAlgReal  b) | isExactRational a && isExactRational b -> Just $ a `compare` b
                                     | True                                   -> Nothing
 
-      -- Structural cases
-      (CMaybe       a, CMaybe    b) -> case (a, b) of
-                                         (Nothing, Nothing) -> Just EQ
-                                         (Nothing, Just{})  -> Just LT
-                                         (Just{},  Nothing) -> Just GT
-                                         (Just av, Just bv) -> case k of
-                                                                 KMaybe ke -> cCompare ke op av bv
-                                                                 _         -> error $ "Unexpected kind in cCompare for maybe's: " ++ show k
-
-      (CEither      a, CEither   b) -> let (kl, kr) = case k of
-                                                        KEither l r -> (l, r)
-                                                        _           -> error $ "Unexpected kind in cCompare for either's: " ++ show k
-                                       in case (a, b) of
-                                            (Left{},   Right{})  -> Just LT
-                                            (Right{},  Left{})   -> Just GT
-                                            (Left av,  Left  bv) -> cCompare kl op av bv
-                                            (Right av, Right bv) -> cCompare kr op av bv
-
-      -- Uninterpreted sorts use the index
-      (CUserSort    a, CUserSort b) -> case (a, b) of
-                                         ((Just i, _), (Just j, _)) -> Just $ i `compare` j
-                                         _                          -> error $ "cCompare: Impossible happened while trying to compare: " ++ show (op, a, b)
-
       -- Lists and tuples use lexicographic ordering
       (CList        a, CList b) -> case k of
                                      KList ke -> lexCmp (map (ke,) a) (map (ke,) b)
@@ -532,9 +511,7 @@ cCompare k op x y =
                            -> case svSetEqual ke a b of
                                  Nothing    -> Nothing  -- We don't know
                                  Just True  -> Just EQ  -- They're equal
-                                 Just False -> Just $ if op `elem` [Equal True, Equal False]
-                                                         then GT  -- Pick GT, So equality    test will fail
-                                                         else EQ  -- Pick EQ, So in-equality test will fail
+                                 Just False -> Just GT  -- Pick GT; so equality test will fail, inequality will pass
                            | True
                            -> error $ "cCompare: Received unexpected set comparison: " ++ show (op, k)
 
@@ -543,16 +520,48 @@ cCompare k op x y =
                            -> case svArrEqual k1 k2 a b of
                                 Nothing    -> Nothing  -- We don't know
                                 Just True  -> Just EQ  -- They're equal
-                                Just False -> Just $ if op `elem` [Equal True, Equal False]
-                                                        then GT  -- Pick GT, So equality    test will fail
-                                                        else EQ  -- Pick EQ, So in-equality test will fail
+                                Just False -> Just GT  -- Pick GT; so equality test will fail, inequality will pass
                            | True
                            -> error $ "cCompare: Received unexpected array comparison: " ++ show (op, k)
 
+
+      -- ADTs. Only equal/inequal on full ADTs. Compares on enumerations.
+      (CADT (s, fks), CADT (s', fks'))
+         -> case k of
+              -- Enumerations. We do a straight comparison on the constructor index
+              KADT _ _ cstrs | all (null . snd) cstrs
+                             -> let cnms = map fst cstrs
+                                in case (s `elemIndex` cnms, s' `elemIndex` cnms) of
+                                     (Just i, Just j) -> Just (i `compare` j)
+                                     r                -> error $ "cCompare: Unable to locate indexes for CADT: " ++ show (k, s, s', r)
+
+              -- Arbitrary ADTs. Only allow equality/inequality
+              _ | op `notElem` [Equal True, Equal False, NotEqual]
+                -> error $ "cCompare: Received unexpected ADT comparison: " ++ show (op, k)
+
+                -- Different constructor
+                | s /= s'
+                -> Just GT -- Pick GT; so equality test will fail, inequality will pass
+
+                -- Same constructor
+                | map fst fks /= map fst fks'
+                -> error $ "cCompare: Mismatching ADT field kinds in comparison: " ++ show (op, k, map fst fks, map fst fks')
+                | True
+                -> let fmatch    = zipWith (\(fk, v1) (_, v2) -> cCompare fk op v1 v2) fks fks'
+                       undecided = any isNothing fmatch   -- Field comparison undecive
+                       allEq     = all (== Just EQ) fmatch -- All fields Equal
+                   in if undecided
+                      then Nothing
+                      else if allEq
+                           then Just EQ
+                           else -- all compared fine, but not all equal
+                                Just GT -- Pick GT; so equality test will fail, inequality will pass
+
+      -- Shouldn't happen:
       _ -> error $ unlines [ ""
                            , "*** Data.SBV.cCompare: Bug in SBV: Unhandled rank in comparison fallthru"
                            , "***"
-                           , "***   Ranks Received: " ++ show (cvRank x, cvRank y)
+                           , "***   Ranks Received: " ++ show (cvRank x, cvRank y, op)
                            , "***"
                            , "*** Please report this as a bug!"
                            ]
@@ -1471,10 +1480,6 @@ svStructuralLessThan x y
    = x `svLessThan` y
    | KTuple{} <- kx
    = tupleLT x y
-   | KMaybe{}  <- kx
-   = maybeLT x y
-   | KEither{} <- kx
-   = eitherLT x y
    | True
    = x `svLessThan` y
    where kx = kindOf x
@@ -1503,53 +1508,6 @@ tupleLT x y = SVal KBool $ Right $ cache res
 
                     svToSV st $ walk $ zipWith chkElt [1..] ks
 
--- | Structural less-than for maybes
-maybeLT :: SVal -> SVal -> SVal
-maybeLT x y = sMaybeCase (       sMaybeCase svFalse (const svTrue)    y)
-                         (\jx -> sMaybeCase svFalse (jx `svStructuralLessThan`) y)
-                         x
-  where ka = case kindOf x of
-               KMaybe k' -> k'
-               k         -> error $ "Data.SBV: Impossible happened, maybeLT called with: " ++ show (k, x, y)
-
-        sMaybeCase brNothing brJust s = SVal KBool $ Right $ cache res
-           where res st = do sv <- svToSV st s
-
-                             let justVal = SVal ka $ Right $ cache $ \_ -> newExpr st ka $ SBVApp MaybeAccess [sv]
-                                 justRes = brJust justVal
-
-                             br1 <- svToSV st brNothing
-                             br2 <- svToSV st justRes
-
-                             -- Do we have a value?
-                             noVal <- newExpr st KBool $ SBVApp (MaybeIs ka False) [sv]
-                             newExpr st KBool $ SBVApp Ite [noVal, br1, br2]
-
--- | Structural less-than for either
-eitherLT :: SVal -> SVal -> SVal
-eitherLT x y = sEitherCase (\lx -> sEitherCase (lx `svStructuralLessThan`) (const svTrue)              y)
-                           (\rx -> sEitherCase (const svFalse)             (rx `svStructuralLessThan`) y)
-                           x
-  where (ka, kb) = case kindOf x of
-                     KEither k1 k2 -> (k1, k2)
-                     k             -> error $ "Data.SBV: Impossible happened, eitherLT called with: " ++ show (k, x, y)
-
-        sEitherCase brA brB sab = SVal KBool $ Right $ cache res
-          where res st = do abv <- svToSV st sab
-
-                            let leftVal  = SVal ka $ Right $ cache $ \_ -> newExpr st ka $ SBVApp (EitherAccess False) [abv]
-                                rightVal = SVal kb $ Right $ cache $ \_ -> newExpr st kb $ SBVApp (EitherAccess True)  [abv]
-
-                                leftRes  = brA leftVal
-                                rightRes = brB rightVal
-
-                            br1 <- svToSV st leftRes
-                            br2 <- svToSV st rightRes
-
-                            --  Which branch are we in? Return the appropriate value:
-                            onLeft <- newExpr st KBool $ SBVApp (EitherIs ka kb False) [abv]
-                            newExpr st KBool $ SBVApp Ite [onLeft, br1, br2]
-
 -- | Convert an 'Data.SBV.SFloat' to an 'Data.SBV.SWord32', preserving the bit-correspondence. Note that since the
 -- representation for @NaN@s are not unique, this function will return a symbolic value when given a
 -- concrete @NaN@.
