@@ -9,7 +9,6 @@
 -- Provable abstraction and the connection to SMT solvers
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE FlexibleInstances     #-}
@@ -17,7 +16,6 @@
 {-# LANGUAGE NamedFieldPuns        #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE TupleSections         #-}
-{-# LANGUAGE UndecidableInstances  #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -29,7 +27,7 @@ module Data.SBV.Provers.Prover (
        , SExecutable(..), isSafe
        , runSMT, runSMTWith
        , SatModel(..), Modelable(..), displayModels, extractModels
-       , getModelDictionaries, getModelValues, getModelUninterpretedValues
+       , getModelDictionaries, getModelValues
        , abc, boolector, bitwuzla, cvc4, cvc5, dReal, mathSAT, yices, z3, openSMT, defaultSMTCfg, defaultDeltaSMTCfg
        , proveWithAny, proveWithAll, proveConcurrentWithAny, proveConcurrentWithAll
        , satWithAny,   satWithAll,   satConcurrentWithAny,   satConcurrentWithAll
@@ -513,11 +511,13 @@ validate reducer isSAT cfg p res =
                                       why s = case s `lookup` S.toList (pgmAssignments (resAsgns result)) of
                                                 Nothing            -> Nothing
                                                 Just (SBVApp o as) -> case o of
-                                                                        Uninterpreted v  -> Just $ "The value depends on the uninterpreted constant " ++ show v ++ "."
                                                                         QuantifiedBool{} -> Just "The value depends on a quantified variable."
                                                                         IEEEFP FP_FMA    -> Just "Floating point FMA operation is not supported concretely."
                                                                         IEEEFP _         -> Just "Not all floating point operations are supported concretely."
                                                                         OverflowOp _     -> Just "Overflow-checking is not done concretely."
+                                                                        Uninterpreted v
+                                                                          | any isADT as -> Just "Models containing ADTs are currently only partially supported."
+                                                                          | True         -> Just $ "The value depends on the uninterpreted constant " ++ show v ++ "."
                                                                         _                -> listToMaybe $ mapMaybe why as
 
                            cstrs = S.toList $ resConstraints result
