@@ -10,7 +10,6 @@
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE NamedFieldPuns      #-}
-{-# LANGUAGE PatternGuards       #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE ViewPatterns        #-}
 
@@ -28,13 +27,13 @@ import           Data.Set             (Set)
 import qualified Data.Set             as Set
 
 import Data.SBV.Core.Data
-import Data.SBV.Core.Kind (smtType, needsFlattening, expandKinds)
+import Data.SBV.Core.Kind (smtType, needsFlattening, expandKinds, substituteADTVars)
 import Data.SBV.Control.Types
 
 import Data.SBV.SMT.Utils
 
 import Data.SBV.Core.Symbolic ( QueryContext(..), SetOp(..), getUserName', getSV, regExpToSMTString, NROp(..)
-                              , SMTDef(..), ResultInp(..), ProgInfo(..), SpecialRelOp(..)
+                              , SMTDef(..), ResultInp(..), ProgInfo(..), SpecialRelOp(..), ADTOp(..)
                               )
 
 import Data.SBV.Utils.PrettyNum (smtRoundingMode, cvToSMTLib)
@@ -43,9 +42,58 @@ import qualified Data.Generics.Uniplate.Data as G
 
 import qualified Data.Graph as DG
 
+
+-- Check that all ADT subkinds are registered. If not, tell the user to do so
+-- NB. This should not be the case as we "automatically" register the subkinds
+-- as we encounter them. But this is mostly a if-something-goes-wrong check.
+checkKinds :: [Kind] -> Maybe String
+checkKinds ks = case [m | m@(n, _) <- apps, n `notElem` defs] of
+                  []       -> Nothing
+                  xs@(f:_) -> let (h, cnt) = case [p | p@(_, i) <- xs, i > 0] of
+                                               (p:_) -> p
+                                               _     -> f
+                                  plu | length xs > 1 = "s are"
+                                      | True          = " is"
+                              in Just $ unlines $ [
+                                    "Data.SBV.mkSymbolic: Impossible happened! Unregistered subkinds."
+                                  , "***"
+                                  , "*** The following kind" ++ plu ++ " not registered: " ++ unwords (map fst xs)
+                                  , "***"
+                                  , "*** Please report this as a bug."
+                                  , "***"
+                                  , "*** As a workaround, you can try registering each ADT subfield, using: "
+                                  , "***"
+                                  , "***    {-# LANGUAGE TypeApplications #-}"
+                                  , "***"
+                                  , "***    import Data.Proxy"
+                                  , "***    registerType (Proxy @" ++ mkProxy h cnt ++ ")"
+                                  ]
+                               ++ extras cnt
+                               ++ [ "***"
+                                  , "*** Even if the workaround does the trick for you, it should not"
+                                  , "*** be needed. Please report this as a bug!"
+                                  ]
+
+
+  where apps = nub [(n, length as) | KApp n as <- concatMap expandKinds ks]
+        defs = nub [n | KADT n _ _ <- ks]
+        mkProxy h 0 = h
+        mkProxy h n = '(' : unwords (h : replicate n "Integer") ++ ")"
+
+        extras 0 = []
+        extras _ = [ "***"
+                   , "*** NB. You can use any base type as arguments, not just 'Integer'."
+                   , "*** It does not need to match the actual use cases, just one instance"
+                   , "*** at some base type is sufficent."
+                   ]
+
 -- | Translate a problem into an SMTLib2 script
 cvt :: SMTLibConverter ([String], [String])
-cvt ctx curProgInfo kindInfo isSat comments allInputs (_, consts) tbls uis defs (SBVPgm asgnsSeq) cstrs out cfg = (pgm, exportedDefs)
+cvt ctx curProgInfo kindInfo isSat comments allInputs (_, consts) tbls uis defs (SBVPgm asgnsSeq) cstrs out cfg
+   | Just s <- checkKinds allKinds
+   = error s
+   | True
+   = (pgm, exportedDefs)
   where allKinds       = Set.toList kindInfo
 
         -- Below can simply be defined as: nub (sort (G.universeBi asgnsSeq))
@@ -62,19 +110,17 @@ cvt ctx curProgInfo kindInfo isSat comments allInputs (_, consts) tbls uis defs
         hasString      = KString     `Set.member` kindInfo
         hasRegExp      = (not . null) [() | (_ :: RegExOp) <- G.universeBi allTopOps]
         hasChar        = KChar      `Set.member` kindInfo
-        hasRounding    = not $ null [s | (s, _) <- usorts, s == "RoundingMode"]
+        hasRounding    = any isRoundingMode allKinds
         hasBVs         = not (null [() | KBounded{} <- allKinds])
-        usorts         = [(s, dt) | KUserSort s dt <- allKinds]
-        trueUSorts     = [s | (s, _) <- usorts, s /= "RoundingMode"]
+        adtsNoRM       = [(s, ps, cs) | k@(KADT s ps cs) <- allKinds, not (isRoundingMode k)]
         tupleArities   = findTupleArities kindInfo
         hasOverflows   = (not . null) [() | (_ :: OvOp) <- G.universeBi allTopOps]
         hasQuantBools  = (not . null) [() | QuantifiedBool{} <- G.universeBi allTopOps]
         hasList        = any isList kindInfo
         hasSets        = any isSet kindInfo
         hasTuples      = not . null $ tupleArities
-        hasEither      = any isEither kindInfo
-        hasMaybe       = any isMaybe  kindInfo
         hasRational    = any isRational kindInfo
+        hasADTs        = not . null $ adtsNoRM
         rm             = roundingMode cfg
         solverCaps     = capabilities (solver cfg)
 
@@ -84,7 +130,7 @@ cvt ctx curProgInfo kindInfo isSat comments allInputs (_, consts) tbls uis defs
         -- Is there a reason why we can't handle this problem?
         -- NB. There's probably a lot more checking we can do here, but this is a start:
         doesntHandle = listToMaybe [nope w | (w, have, need) <- checks, need && not (have solverCaps)]
-           where checks = [ ("data types",             supportsDataTypes,          hasTuples || hasEither || hasMaybe)
+           where checks = [ ("data types",             supportsDataTypes,          hasTuples || hasADTs)
                           , ("set operations",         supportsSets,               hasSets)
                           , ("bit vectors",            supportsBitVectors,         hasBVs)
                           , ("special relations",      supportsSpecialRels,        needsSpecialRels)
@@ -92,7 +138,7 @@ cvt ctx curProgInfo kindInfo isSat comments allInputs (_, consts) tbls uis defs
                           , ("unbounded integers",     supportsUnboundedInts,      hasInteger)
                           , ("algebraic reals",        supportsReals,              hasReal)
                           , ("floating-point numbers", supportsIEEE754,            hasFP)
-                          , ("uninterpreted sorts",    supportsUninterpretedSorts, not (null trueUSorts))
+                          , ("has data-types/sorts",   supportsADTs,               not (null adtsNoRM))
                           ]
 
                  nope w = [ "***     Given problem requires support for " ++ w
@@ -146,11 +192,9 @@ cvt ctx curProgInfo kindInfo isSat comments allInputs (_, consts) tbls uis defs
            | hasInteger            = setAll "has unbounded values"
            | hasRational           = setAll "has rational values"
            | hasReal               = setAll "has algebraic reals"
-           | not (null trueUSorts) = setAll "has user-defined sorts"
+           | hasADTs               = setAll "has user-defined data-types"
            | hasNonBVArrays        = setAll "has non-bitvector arrays"
            | hasTuples             = setAll "has tuples"
-           | hasEither             = setAll "has either type"
-           | hasMaybe              = setAll "has maybe type"
            | hasSets               = setAll "has sets"
            | hasList               = setAll "has lists"
            | hasChar               = setAll "has chars"
@@ -216,14 +260,12 @@ cvt ctx curProgInfo kindInfo isSat comments allInputs (_, consts) tbls uis defs
 
         pgm  =  map ("; " ++) comments
              ++ settings
-             ++ [ "; --- uninterpreted sorts ---" ]
-             ++ concatMap declSort usorts
              ++ [ "; --- tuples ---" ]
              ++ concatMap declTuple tupleArities
              ++ [ "; --- sums ---" ]
-             ++ (if containsSum       kindInfo then declSum       else [])
-             ++ (if containsMaybe     kindInfo then declMaybe     else [])
              ++ (if containsRationals kindInfo then declRationals else [])
+             ++ [ "; --- ADTs  --- " | not (null adtsNoRM)]
+             ++ declADT adtsNoRM
              ++ [ "; --- literal constants ---" ]
              ++ concatMap (declConst cfg) consts
              ++ [ "; --- top level inputs ---"]
@@ -299,20 +341,48 @@ cvt ctx curProgInfo kindInfo isSat comments allInputs (_, consts) tbls uis defs
                         Just u  | show s /= u -> Just $ "tracks user variable " ++ show u
                         _                     -> Nothing
 
--- | Declare new sorts
-declSort :: (String, Maybe [String]) -> [String]
-declSort (s, _)
-  | s == "RoundingMode" -- built-in-sort; so don't declare.
-  = []
-declSort (s, Nothing) = [ "(declare-sort " ++ s ++ " 0)  ; N.B. Uninterpreted sort."
-                        , "(declare-fun " ++ witnessName s ++ " () " ++ s ++ ")"
-                        ]
-declSort (s, Just fs) = [ "(declare-datatypes ((" ++ s ++ " 0)) ((" ++ unwords (map (\c -> "(" ++ c ++ ")") fs) ++ ")))"
-                        , "(define-fun " ++ s ++ "_constrIndex ((x " ++ s ++ ")) Int"
-                        ] ++ ["   " ++ body fs (0::Int)] ++ [")"]
-        where body []     _ = ""
-              body [_]    i = show i
-              body (c:cs) i = "(ite (= x " ++ c ++ ") " ++ show i ++ " " ++ body cs (i+1) ++ ")"
+-- | Declare ADTs
+declADT :: [(String, [(String, Kind)], [(String, [Kind])])] -> [String]
+declADT = concatMap declGroup . DG.stronglyConnComp . map mkNode
+  where mkNode adt@(n, pks, cstrs) = (adt, n, [s | KApp s _ <- concatMap expandKinds (map snd pks ++ concatMap snd cstrs)])
+
+        declGroup (DG.AcyclicSCC d )  = singleADT d
+        declGroup (DG.CyclicSCC  ds)
+            = case ds of
+                []  -> error "Data.SBV.declADT: Impossible happened: an empty cyclic group was returned!"
+                [d] -> singleADT d
+                _   -> multiADT ds
+
+        parParens :: [(String, Kind)] -> (String, String)
+        parParens [] = ("", "")
+        parParens ps = (" (par (" ++ unwords (map fst ps) ++ ")", ")")
+
+        mkC (nm, []) = nm
+        mkC (nm, ts) = nm ++ " " ++ unwords ['(' : mkF (nm ++ "_" ++ show i) t ++ ")" | (i, t) <- zip [(1::Int)..] ts]
+          where mkF a t  = "get" ++ a ++ " " ++ smtType t
+
+        singleADT :: (String, [(String, Kind)], [(String, [Kind])]) -> [String]
+        singleADT (tName, [], []) = ["(declare-sort " ++ tName ++ " 0) ; N.B. Uninterpreted sort."]
+        singleADT (tName, pks, cstrs) = ("; User defined ADT: " ++ tName) : decl
+          where decl =  ("(declare-datatype " ++ tName ++ parOpen ++ " (")
+                     :  ["    (" ++ mkC c ++ ")" | c <- cstrs]
+                     ++ ["))" ++ parClose]
+
+                (parOpen, parClose) = parParens pks
+
+        multiADT :: [(String, [(String, Kind)], [(String, [Kind])])] -> [String]
+        multiADT adts = ("; User defined mutually-recursive ADTs: " ++ intercalate ", " (map (\(a, _, _) -> a) adts)) : decl
+          where decl = ("(declare-datatypes (" ++ typeDecls ++ ") (")
+                     : concatMap adtBody adts
+                    ++ ["))"]
+
+                typeDecls = unwords ['(' : name ++ " " ++ show (length pks) ++ ")" | (name, pks, _) <- adts]
+
+                adtBody (_, pks, cstrs) = body
+                  where (parOpen, parClose) = parParens pks
+                        body =  ("    " ++ parOpen ++ " (")
+                             :  ["        (" ++ mkC c ++ ")" | c <- cstrs]
+                             ++ ["     )" ++ parClose]
 
 -- | Declare tuple datatypes
 --
@@ -351,30 +421,10 @@ findTupleArities ks = Set.toAscList
                     $ Set.map length
                     $ Set.fromList [ tupKs | KTuple tupKs <- Set.toList ks ]
 
--- | Is @Either@ being used?
-containsSum :: Set Kind -> Bool
-containsSum = not . Set.null . Set.filter isEither
-
--- | Is @Maybe@ being used?
-containsMaybe :: Set Kind -> Bool
-containsMaybe = not . Set.null . Set.filter isMaybe
-
 -- | Is @Rational@ being used?
 containsRationals :: Set Kind -> Bool
 containsRationals = not . Set.null . Set.filter isRational
 
-declSum :: [String]
-declSum = [ "(declare-datatypes ((SBVEither 2)) ((par (T1 T2)"
-          , "                                    ((left_SBVEither  (get_left_SBVEither  T1))"
-          , "                                     (right_SBVEither (get_right_SBVEither T2))))))"
-          ]
-
-declMaybe :: [String]
-declMaybe = [ "(declare-datatypes ((SBVMaybe 1)) ((par (T)"
-            , "                                    ((nothing_SBVMaybe)"
-            , "                                     (just_SBVMaybe (get_just_SBVMaybe T))))))"
-            ]
-
 -- Internally, we do *not* keep the rationals in reduced form! So, the boolean operators explicitly do the math
 -- to make sure equivalent values are treated correctly.
 declRationals :: [String]
@@ -400,12 +450,9 @@ cvtInc curProgInfo inps newKs (_, consts) tbls uis (SBVPgm asgnsSeq) cstrs cfg =
             -- any new settings?
                settings
             -- sorts
-            ++ concatMap declSort [(s, dt) | KUserSort s dt <- newKinds]
+            ++ declADT [(s, pks, cs) | k@(KADT s pks cs) <- newKinds, not (isRoundingMode k)]
             -- tuples. NB. Only declare the new sizes, old sizes persist.
             ++ concatMap declTuple (findTupleArities newKs)
-            -- sums
-            ++ (if containsSum   newKs then declSum   else [])
-            ++ (if containsMaybe newKs then declMaybe else [])
             -- constants
             ++ concatMap (declConst cfg) consts
             -- inputs
@@ -714,11 +761,6 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
         lift2Cmp o fo | fpOp = lift2 fo
                       | True = lift2 o
 
-        unintComp o [a, b]
-          | KUserSort s (Just _) <- kindOf (hd "unintComp" arguments)
-          = let idx v = "(" ++ s ++ "_constrIndex " ++ v ++ ")" in "(" ++ o ++ " " ++ idx a ++ " " ++ idx b ++ ")"
-        unintComp o sbvs = error $ "SBV.SMT.SMTLib2.sh.unintComp: Unexpected arguments: "   ++ show (o, sbvs, map kindOf arguments)
-
         stringOrChar KString = True
         stringOrChar KChar   = True
         stringOrChar _       = False
@@ -740,49 +782,30 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
         lift1  o _ [x]    = "(" ++ o ++ " " ++ x ++ ")"
         lift1  o _ sbvs   = error $ "SBV.SMT.SMTLib2.sh.lift1: Unexpected arguments: "   ++ show (o, sbvs)
 
-        -- We fully qualify the constructor with their types to work around type checking issues
-        -- Note that this is rather bizarre, as we're tagging the constructor with its *result* type,
-        -- not its full function type as one would expect. But this is per the spec: Pg. 27 of SMTLib 2.6 spec
-        -- says:
-        --
-        --    To simplify sort checking, a function symbol in a term can be annotated with one of its result sorts sigma.
-        --
-        -- I wish it was the full type here not just the result, but we go with the spec. Also see: <http://github.com/Z3Prover/z3/issues/2135>
-        -- and in particular <http://github.com/Z3Prover/z3/issues/2135#issuecomment-477636435>
-        dtConstructor fld args res = "((as " ++ fld ++ " " ++ smtType res ++ ") " ++ unwords (map cvtSV args) ++ ")"
-
-        -- Similarly, we fully qualify the accessors with their types to work around type checking issues
-        -- Unfortunately, z3 and CVC4 are behaving differently, so we tie this ascription to a solver capability.
-        dtAccessor fld params res
-           | supportsDirectAccessors caps = dResult
-           | True                         = aResult
-          where dResult = "(_ is " ++ fld ++ ")"
-                ps      = " (" ++ unwords (map smtType params) ++ ") "
-                aResult = "(_ is (" ++ fld ++ ps ++ smtType res ++ "))"
-
         sh (SBVApp Ite [a, b, c]) = "(ite " ++ cvtSV a ++ " " ++ cvtSV b ++ " " ++ cvtSV c ++ ")"
 
         sh (SBVApp (LkUp (t, aKnd, _, l) i e) [])
           | needsCheck = "(ite " ++ cond ++ cvtSV e ++ " " ++ lkUp ++ ")"
           | True       = lkUp
-          where needsCheck = case aKnd of
+          where unexpected = error $ "SBV.SMT.SMTLib2.cvtExp: Unexpected: " ++ show aKnd
+                needsCheck = case aKnd of
+                              KVar{}        -> unexpected
                               KBool         -> (2::Integer) > fromIntegral l
                               KBounded _ n  -> (2::Integer)^n > fromIntegral l
                               KUnbounded    -> True
-                              KUserSort s _ -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected uninterpreted valued index: " ++ s
-                              KReal         -> error "SBV.SMT.SMTLib2.cvtExp: unexpected real valued index"
-                              KFloat        -> error "SBV.SMT.SMTLib2.cvtExp: unexpected float valued index"
-                              KDouble       -> error "SBV.SMT.SMTLib2.cvtExp: unexpected double valued index"
-                              KFP{}         -> error "SBV.SMT.SMTLib2.cvtExp: unexpected arbitrary float valued index"
-                              KRational{}   -> error "SBV.SMT.SMTLib2.cvtExp: unexpected rational valued index"
-                              KChar         -> error "SBV.SMT.SMTLib2.cvtExp: unexpected char valued index"
-                              KString       -> error "SBV.SMT.SMTLib2.cvtExp: unexpected string valued index"
-                              KList k       -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected list valued: " ++ show k
-                              KSet  k       -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected set valued: " ++ show k
-                              KTuple k      -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected tuple valued: " ++ show k
-                              KMaybe k      -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected maybe valued: " ++ show k
-                              KEither k1 k2 -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected sum valued: " ++ show (k1, k2)
-                              KArray  k1 k2 -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected array valued: " ++ show (k1, k2)
+                              KApp _ _      -> unexpected
+                              KADT _ _ _    -> unexpected
+                              KReal         -> unexpected
+                              KFloat        -> unexpected
+                              KDouble       -> unexpected
+                              KFP _ _       -> unexpected
+                              KRational     -> unexpected
+                              KChar         -> unexpected
+                              KString       -> unexpected
+                              KList _       -> unexpected
+                              KSet  _       -> unexpected
+                              KTuple _      -> unexpected
+                              KArray  _ _   -> unexpected
 
                 lkUp = "(" ++ getTable tableMap t ++ " " ++ cvtSV i ++ ")"
 
@@ -791,6 +814,7 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
                  | True      = gtl ++ " "
 
                 (less, leq) = case aKnd of
+                                KVar{}        -> error "SBV.SMT.SMTLib2.cvtExp: unexpected variable index"
                                 KBool         -> error "SBV.SMT.SMTLib2.cvtExp: unexpected boolean valued index"
                                 KBounded{}    -> if hasSign i then ("bvslt", "bvsle") else ("bvult", "bvule")
                                 KUnbounded    -> ("<", "<=")
@@ -801,12 +825,11 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
                                 KFP{}         -> ("fp.lt", "fp.leq")
                                 KChar         -> error "SBV.SMT.SMTLib2.cvtExp: unexpected string valued index"
                                 KString       -> error "SBV.SMT.SMTLib2.cvtExp: unexpected string valued index"
-                                KUserSort s _ -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected uninterpreted valued index: " ++ s
+                                KApp  s _     -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected ADT applied index: " ++ s
+                                KADT  s _ _   -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected ADT valued index: " ++ s
                                 KList k       -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected sequence valued index: " ++ show k
                                 KSet  k       -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected set valued index: " ++ show k
                                 KTuple k      -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected tuple valued index: " ++ show k
-                                KMaybe k      -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected maybe valued index: " ++ show k
-                                KEither k1 k2 -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected sum valued index: " ++ show (k1, k2)
                                 KArray  k1 k2 -> error $ "SBV.SMT.SMTLib2.cvtExp: unexpected array valued index: " ++ show (k1, k2)
 
                 mkCnst = cvtCV rm . mkConstCV (kindOf i)
@@ -822,6 +845,8 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
         sh (SBVApp (Uninterpreted nm) [])   = nm
         sh (SBVApp (Uninterpreted nm) args) = "(" ++ nm ++ " " ++ unwords (map cvtSV args) ++ ")"
 
+        sh (SBVApp (ADTOp aop) args) = handleADT caps aop args
+
         sh (SBVApp (QuantifiedBool i) [])   = i
         sh (SBVApp (QuantifiedBool i) args) = error $ "SBV.SMT.SMTLib2.cvtExp: unexpected arguments to quantified boolean: " ++ show (i, args)
 
@@ -904,7 +929,7 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
         sh (SBVApp (OverflowOp op) args) = "(" ++ show op ++ " " ++ unwords (map cvtSV args) ++ ")"
 
         -- Note the unfortunate reversal in StrInRe..
-        sh (SBVApp (StrOp (StrInRe r)) args) = "(str.in.re " ++ unwords (map cvtSV args) ++ " " ++ regExpToSMTString r ++ ")"
+        sh (SBVApp (StrOp (StrInRe r)) args) = "(str.in_re " ++ unwords (map cvtSV args) ++ " " ++ regExpToSMTString r ++ ")"
         sh (SBVApp (StrOp op)          args) = "(" ++ show op ++ " " ++ unwords (map cvtSV args) ++ ")"
 
         sh (SBVApp (RegExOp o@RegExEq{})  []) = show o
@@ -929,21 +954,8 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
         sh (SBVApp (TupleConstructor n)   args)  = "((as mkSBVTuple" ++ show n ++ " " ++ smtType (KTuple (map kindOf args)) ++ ") " ++ unwords (map cvtSV args) ++ ")"
         sh (SBVApp (TupleAccess      i n) [tup]) = "(proj_" ++ show i ++ "_SBVTuple" ++ show n ++ " " ++ cvtSV tup ++ ")"
 
-        sh (SBVApp (EitherConstructor k1 k2 False) [arg]) =       dtConstructor "left_SBVEither"  [arg] (KEither k1 k2)
-        sh (SBVApp (EitherConstructor k1 k2 True ) [arg]) =       dtConstructor "right_SBVEither" [arg] (KEither k1 k2)
-        sh (SBVApp (EitherIs          k1 k2 False) [arg]) = '(' : dtAccessor    "left_SBVEither"  [k1]  (KEither k1 k2) ++ " " ++ cvtSV arg ++ ")"
-        sh (SBVApp (EitherIs          k1 k2 True ) [arg]) = '(' : dtAccessor    "right_SBVEither" [k2]  (KEither k1 k2) ++ " " ++ cvtSV arg ++ ")"
-        sh (SBVApp (EitherAccess            False) [arg]) = "(get_left_SBVEither "  ++ cvtSV arg ++ ")"
-        sh (SBVApp (EitherAccess            True ) [arg]) = "(get_right_SBVEither " ++ cvtSV arg ++ ")"
-
         sh (SBVApp  RationalConstructor    [t, b]) = "(SBV.Rational " ++ cvtSV t ++ " " ++ cvtSV b ++ ")"
 
-        sh (SBVApp (MaybeConstructor k False) [])    =       dtConstructor "nothing_SBVMaybe" []    (KMaybe k)
-        sh (SBVApp (MaybeConstructor k True)  [arg]) =       dtConstructor "just_SBVMaybe"    [arg] (KMaybe k)
-        sh (SBVApp (MaybeIs          k False) [arg]) = '(' : dtAccessor    "nothing_SBVMaybe" []    (KMaybe k) ++ " " ++ cvtSV arg ++ ")"
-        sh (SBVApp (MaybeIs          k True ) [arg]) = '(' : dtAccessor    "just_SBVMaybe"    [k]   (KMaybe k) ++ " " ++ cvtSV arg ++ ")"
-        sh (SBVApp MaybeAccess                [arg]) = "(get_just_SBVMaybe " ++ cvtSV arg ++ ")"
-
         sh (SBVApp Implies [a, b]) = "(=> " ++ cvtSV a ++ " " ++ cvtSV b ++ ")"
 
         sh inp@(SBVApp op args)
@@ -966,25 +978,16 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
           | Just f <- lookup op uninterpretedTable
           = f (map cvtSV args)
           | True
-          = if not (null args) && isUserSort (hd "isUserSort" args)
-            then error $ unlines [ ""
-                                 , "*** Cannot translate operator        : " ++ show op
-                                 , "*** When applied to arguments of kind: " ++ intercalate ", " (nub (map (show . kindOf) args))
-                                 , "*** Found as part of the expression  : " ++ show inp
-                                 , "***"
-                                 , "*** Note that uninterpreted kinds only support equality."
-                                 , "*** If you believe this is in error, please report!"
-                                 ]
-            else error $ unlines [ ""
-                                 , "*** SBV.SMT.SMTLib2.cvtExp.sh: impossible happened; can't translate: " ++ show inp
-                                 , "***"
-                                 , "*** Applied to arguments of type: " ++ intercalate ", " (nub (map (show . kindOf) args))
-                                 , "***"
-                                 , "*** This can happen if the Num instance isn't properly defined for a lifted kind."
-                                 , "*** (See https://github.com/LeventErkok/sbv/issues/698 for a discussion.)"
-                                 , "***"
-                                 , "*** If you believe this is in error, please report!"
-                                 ]
+          = error $ unlines [ ""
+                            , "*** SBV.SMT.SMTLib2.cvtExp.sh: impossible happened; can't translate: " ++ show inp
+                            , "***"
+                            , "*** Applied to arguments of type: " ++ intercalate ", " (nub (map (show . kindOf) args))
+                            , "***"
+                            , "*** This can happen if the Num instance isn't properly defined for a lifted kind."
+                            , "*** (See https://github.com/LeventErkok/sbv/issues/698 for a discussion.)"
+                            , "***"
+                            , "*** If you believe this is in error, please report!"
+                            ]
           where smtOpBVTable  = [ (Plus,          lift2   "bvadd")
                                 , (Minus,         lift2   "bvsub")
                                 , (Times,         lift2   "bvmul")
@@ -1051,10 +1054,6 @@ cvtExp cfg curProgInfo caps rm tableMap expr@(SBVApp _ arguments) = sh expr
                 uninterpretedTable = [ (Equal True,  lift2S "="        "="        True)
                                      , (Equal False, lift2S "="        "="        True)
                                      , (NotEqual,    liftNS "distinct" "distinct" True)
-                                     , (LessThan,    unintComp "<")
-                                     , (GreaterThan, unintComp ">")
-                                     , (LessEq,      unintComp "<=")
-                                     , (GreaterEq,   unintComp ">=")
                                      ]
 
                 -- For strings, equality and comparisons are the only operators
@@ -1085,7 +1084,7 @@ declareFun = declareName . show
 -- Otherwise, we just declare the name
 declareName :: String -> SBVType -> Maybe String -> [String]
 declareName s t@(SBVType inputKS) mbCmnt = decl : restrict
-  where decl        = "(declare-fun " ++ s ++ " " ++ cvtType t ++ ")" ++ maybe "" (" ; " ++) mbCmnt
+  where decl = "(declare-fun " ++ s ++ " " ++ cvtType t ++ ")" ++ maybe "" (" ; " ++) mbCmnt
 
         (args, result) = case inputKS of
                           [] -> error $ "SBV.declareName: Unexpected empty type for: " ++ show s
@@ -1135,11 +1134,12 @@ declareName s t@(SBVType inputKS) mbCmnt = decl : restrict
         mkAnd cs  context = context $ "(and " ++ unwords cs ++ ")"
 
         walk :: Int -> String -> (Kind -> String -> [String]) -> Kind -> [String]
+        walk _d nm f k@KVar      {}         = f k nm
         walk _d nm f k@KBool     {}         = f k nm
         walk _d nm f k@KBounded  {}         = f k nm
         walk _d nm f k@KUnbounded{}         = f k nm
         walk _d nm f k@KReal     {}         = f k nm
-        walk _d nm f k@KUserSort {}         = f k nm
+        walk _d nm f k@KApp      {}         = f k nm
         walk _d nm f k@KFloat    {}         = f k nm
         walk _d nm f k@KDouble   {}         = f k nm
         walk _d nm f k@KRational {}         = f k nm
@@ -1160,18 +1160,14 @@ declareName s t@(SBVType inputKS) mbCmnt = decl : restrict
                                                 project i = "(proj_" ++ show i ++ "_" ++ tt ++ " " ++ nm ++ ")"
                                                 nmks      = [(project i, k) | (i, k) <- zip [1::Int ..] ks]
                                             in concatMap (\(n, k) -> walk (d+1) n f k) nmks
-        walk  d  nm  f km@(KMaybe k)      = let n = "(get_just_SBVMaybe " ++ nm ++ ")"
-                                            in  ["(=> " ++ "((_ is (just_SBVMaybe (" ++ smtType k ++ ") " ++ smtType km ++ ")) " ++ nm ++ ") " ++ c ++ ")" | c <- walk (d+1) n f k]
-        walk  d  nm  f ke@(KEither k1 k2) = let n1 = "(get_left_SBVEither "  ++ nm ++ ")"
-                                                n2 = "(get_right_SBVEither " ++ nm ++ ")"
-                                                c1 = ["(=> " ++ "((_ is (left_SBVEither ("  ++ smtType k1 ++ ") " ++ smtType ke ++ ")) " ++ nm ++ ") " ++ c ++ ")" | c <- walk (d+1) n1 f k1]
-                                                c2 = ["(=> " ++ "((_ is (right_SBVEither (" ++ smtType k2 ++ ") " ++ smtType ke ++ ")) " ++ nm ++ ") " ++ c ++ ")" | c <- walk (d+1) n2 f k2]
-                                            in c1 ++ c2
         walk d  nm f  (KArray k1 k2)
           | all charRatFree [k1, k2]      = []
           | True                          = let fnm   = "array" ++ show d
                                                 cstrs = walk (d+1) ("(select " ++ nm ++ " " ++ fnm ++ ")") f k2
                                             in mkAnd cstrs $ \hole -> ["(forall ((" ++ fnm ++ " " ++ smtType k1 ++ ")) " ++ hole ++ ")"]
+        walk d nm f (KADT ty dict pureFS) = let fs = [(c, map (substituteADTVars ty dict) ks) | (c, ks) <- pureFS]
+                                                nmks  = [("(get" ++ c ++ "_" ++ show i ++ " " ++ nm ++ ")", k) | (c, ks) <- fs, (i, k) <- zip [(1::Int)..] ks]
+                                            in concatMap (\(n, k) -> walk (d+1) n f k) nmks
 
 -----------------------------------------------------------------------------------------------
 -- Casts supported by SMTLib. (From: <https://smt-lib.org/theories-FloatingPoint.shtml>)
@@ -1241,6 +1237,20 @@ shft :: String -> String -> SV -> SV -> String
 shft oW oS x c = "(" ++ o ++ " " ++ cvtSV x ++ " " ++ cvtSV c ++ ")"
    where o = if hasSign x then oS else oW
 
+-- ADT operations
+handleADT :: SolverCapabilities -> ADTOp -> [SV] -> String
+handleADT caps op args = case args of
+                          [] -> f
+                          _  -> "(" ++ f ++ " " ++ unwords (map cvtSV args) ++ ")"
+  where f = case op of
+              ADTConstructor nm k -> ascribe nm k
+              ADTTester      nm k -> if supportsDirectTesters caps
+                                     then nm
+                                     else ascribe nm k
+              ADTAccessor    nm _ -> nm
+
+        ascribe nm k = "(as " ++ nm ++ " " ++ smtType k ++ ")"
+
 -- Various casts
 handleKindCast :: Kind -> Kind -> String -> String
 handleKindCast kFrom kTo a
@@ -1340,3 +1350,5 @@ setSMTOption cfg = set
         smtBool :: Bool -> String
         smtBool True  = "true"
         smtBool False = "false"
+
+{- HLint ignore module "Use record patterns" -}
