@@ -14,8 +14,8 @@
 {-# LANGUAGE CPP                 #-}
 {-# LANGUAGE DataKinds           #-}
 {-# LANGUAGE OverloadedLists     #-}
-{-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE QuasiQuotes         #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeAbstractions    #-}
 {-# LANGUAGE TypeApplications    #-}
 
@@ -139,7 +139,7 @@ initsLength :: forall a. SymVal a => TP (Proof (Forall "xs" [a] -> SBool))
 initsLength =
    sInduct "initsLength"
            (\(Forall xs) -> length (inits xs) .== 1 + length xs)
-           (length @a) $
+           (length @a, []) $
            \ih xs -> [] |- length (inits xs)
                         ?? ih
                         =: 1 + length xs
@@ -372,7 +372,7 @@ enumLen :: TP (Proof (Forall "n" Integer -> Forall "m" Integer -> SBool))
 enumLen =
   sInduct "enumLen"
           (\(Forall n) (Forall m) -> length [sEnum|n .. m|] .== 0 `smax` (m - n + 1))
-          (\n m -> abs (m - n)) $
+          (\n m -> abs (m - n), []) $
           \ih n m -> [] |- length [sEnum|n+1 .. m|]
                         =: cases [ n+1 .>  m ==> trivial
                                  , n+1 .<= m ==> length (n+1 .: [sEnum|n+2 .. m|])
@@ -411,7 +411,7 @@ revNM = do
   helper <- sInduct "helper"
                     (\(Forall @"m" (m :: SInteger)) (Forall @"n" n) ->
                           n .< m .=> [sEnum|m, m-1 .. n+1|] ++ [n] .== [sEnum|m, m-1 .. n|])
-                    (\m n -> abs (m - n)) $
+                    (\m n -> abs (m - n), []) $
                     \ih m n -> [n .< m] |- [sEnum|m, m-1 .. n+1|] ++ [n]
                                         =: m .: [sEnum|m-1, m-2 .. n+1|] ++ [n]
                                         ?? ih
@@ -421,7 +421,7 @@ revNM = do
 
   sInduct "revNM"
           (\(Forall n) (Forall m) -> reverse [sEnum|n .. m|] .== [sEnum|m, m-1 .. n|])
-          (\n m -> abs (m - n)) $
+          (\n m -> abs (m - n), []) $
           \ih n m -> [] |- reverse [sEnum|n .. m|]
                         =: cases [ n .>  m ==> trivial
                                  , n .<= m ==> reverse (n .: [sEnum|(n+1) .. m|])
@@ -1140,7 +1140,7 @@ destutterIdempotent = do
    -- Helper: prove that noAdd is true for the result of destutter
    helper3 <- sInductWith cvc5 "helper3"
                   (\(Forall @"xs" (xs :: SList a)) -> noAdd (destutter xs))
-                  length $
+                  (length, []) $
                   \ih xs -> []
                          |- noAdd (destutter xs)
                          =: split xs
@@ -1693,7 +1693,7 @@ interleave = smtFunction "interleave" (\xs ys -> ite (null  xs) ys (head xs .: i
 interleaveLen :: forall a. SymVal a => TP (Proof (Forall "xs" [a] -> Forall "ys" [a] -> SBool))
 interleaveLen = sInduct "interleaveLen"
                         (\(Forall xs) (Forall ys) -> length xs + length ys .== length (interleave xs ys))
-                        (\xs ys -> length xs + length ys) $
+                        (\xs ys -> length xs + length ys, []) $
                         \ih xs ys -> [] |- length xs + length ys .== length (interleave xs ys)
                                         =: split xs
                                                  trivial
@@ -1752,7 +1752,7 @@ interleaveRoundTrip = do
          (\(Forall @"xs" xs) (Forall @"ys" ys) (Forall @"alts" alts) ->
                length xs .== length ys .=> let (es, os) = untuple alts
                                            in uninterleaveGen (interleave xs ys) alts .== tuple (reverse es ++ xs, reverse os ++ ys))
-         (\xs ys _alts -> length xs + length ys) $
+         (\xs ys _alts -> length xs + length ys, []) $
          \ih xs ys alts -> [length xs .== length ys]
                         |- let (es, os) = untuple alts
                         in uninterleaveGen (interleave xs ys) alts
