@@ -9,14 +9,10 @@
 -- Proves a simple mutex algorithm correct up to a given bound.
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
-{-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE FlexibleInstances   #-}
-{-# LANGUAGE OverloadedLists     #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TypeApplications    #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -29,10 +25,10 @@ import Data.SBV.Control
 data State = Idle     -- ^ Regular work
            | Ready    -- ^ Intention to enter critical state
            | Critical -- ^ In the critical state
-           deriving (Enum, Bounded)
+           deriving Show
 
 -- | Make 'State' a symbolic enumeration
-mkSymbolicEnumeration ''State
+mkSymbolic [''State]
 
 -- | The mutex property holds for two sequences of state transitions, if they are not in
 -- their critical section at the same time.
