@@ -28,12 +28,9 @@
 -- case, the second student would be right.
 ------------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
-{-# LANGUAGE FlexibleInstances   #-}
-{-# LANGUAGE OverloadedStrings   #-}
-{-# LANGUAGE StandaloneDeriving  #-}
-{-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE FlexibleInstances  #-}
+{-# LANGUAGE TemplateHaskell    #-}
+{-# LANGUAGE TypeApplications   #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -43,10 +40,9 @@ import Data.SBV
 
 -- | Colors of the flowers
 data Color = Red | Yellow | Blue
-          deriving (Enum, Bounded)
 
 -- | Make 'Color' a symbolic value.
-mkSymbolicEnumeration ''Color
+mkSymbolic [''Color]
 
 -- | Represent flowers by symbolic integers
 type Flower = SInteger
