@@ -36,13 +36,9 @@
 -- to the final one.
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
-{-# LANGUAGE FlexibleInstances   #-}
-{-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
-{-# LANGUAGE TemplateHaskell     #-}
-{-# LANGUAGE TypeApplications    #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE TemplateHaskell   #-}
+{-# LANGUAGE TypeApplications  #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -55,10 +51,9 @@ import Data.Proxy
 
 -- | Colors we're allowed
 data Color = Black | Blue | Green | Red
-          deriving (Enum, Bounded)
 
 -- | Make 'Color' a symbolic value.
-mkSymbolicEnumeration ''Color
+mkSymbolic [''Color]
 
 -- | Use 8-bit words for button numbers, even though we only have 1 to 19.
 type Button  = Word8
