@@ -71,7 +71,7 @@ gcdNonNegative = do
      -- We first prove over nGCD, using strong induction with the measure @a+b@.
      nn <- sInduct "nonNegativeNGCD"
                    (\(Forall a) (Forall b) -> a .>= 0 .&& b .>= 0 .=> nGCD a b .>= 0)
-                   (\_a b -> b) $
+                   (\_a b -> b, []) $
                    \ih a b -> [a .>= 0, b .>= 0]
                            |- cases [ b .== 0 ==> trivial
                                     , b ./= 0 ==> nGCD a b .>= 0
@@ -106,7 +106,7 @@ gcdZero = do
   nGCDZero <-
     sInduct "nGCDZero"
             (\(Forall @"a" a) (Forall @"b" b) -> a .>= 0 .&& b .>= 0 .&& nGCD a b .== 0 .=> a .== 0 .&& b .== 0)
-            (\_a b -> b) $
+            (\_a b -> b, []) $
             \ih a b -> [a .>= 0, b .>= 0]
                     |- (nGCD a b .== 0 .=> a .== 0 .&& b .== 0)
                     =: cases [ b .== 0 ==> trivial
@@ -129,6 +129,7 @@ gcdZero = do
 --   Result:                               Q.E.D.
 -- Lemma: commutative
 --   Step: 1                               Q.E.D.
+--   Step: 2                               Q.E.D.
 --   Result:                               Q.E.D.
 -- [Proven] commutative :: Ɐa ∷ Integer → Ɐb ∷ Integer → Bool
 commutative :: TP (Proof (Forall "a" Integer -> Forall "b" Integer -> SBool))
@@ -148,7 +149,8 @@ commutative = do
     calc "commutative"
           (\(Forall a) (Forall b) -> gcd a b .== gcd b a) $
           \a b -> [] |- gcd a b
-                     ?? nGCDComm
+                     =: nGCD (abs a) (abs b)
+                     ?? nGCDComm `at` (Inst @"a" (abs a), Inst @"b" (abs b))
                      =: gcd b a
                      =: qed
 
@@ -293,7 +295,7 @@ dvdEvenWhenOdd :: TP (Proof (Forall "d" Integer -> Forall "a" Integer -> SBool))
 dvdEvenWhenOdd = calc "dvdEvenWhenOdd"
                       (\(Forall d) (Forall a) -> isOdd d .&& d `dvd` (2*a) .=> d `dvd` a) $
                       \d a ->  [isOdd d, d `dvd` (2*a)]
-                           |-> let t = (d - 1) `sEDiv` 2
+                           |-  let t = (d - 1) `sEDiv` 2
                                    m = (2*a)   `sEDiv` d
                             in sTrue
 
@@ -422,7 +424,7 @@ gcdDivides = do
    -- Use strong induction to prove divisibility over non-negative numbers.
    dNGCD <- sInduct "dvdNGCD"
                      (\(Forall @"a" a) (Forall @"b" b) -> a .>= 0 .&& b .>= 0 .=> nGCD a b `dvd` a .&& nGCD a b `dvd` b)
-                     (\_a b -> b) $
+                     (\_a b -> b, []) $
                      \ih a b -> [a .>= 0, b .>= 0]
                              |- let g = nGCD a b
                              in g `dvd` a .&& g `dvd` b
@@ -447,6 +449,7 @@ gcdDivides = do
 -- ==== __Proof__
 -- >>> runTP gcdMaximal
 -- Lemma: dvdAbs                           Q.E.D.
+-- Lemma: gcdComm                          Q.E.D.
 -- Lemma: eDiv                             Q.E.D.
 -- Lemma: helper
 --   Step: 1 (x `dvd` a && x `dvd` b)      Q.E.D.
@@ -467,13 +470,16 @@ gcdDivides = do
 --     Step: 1.1.2                         Q.E.D.
 --     Step: 1.2.1                         Q.E.D.
 --     Step: 1.2.2                         Q.E.D.
+--     Step: 1.2.3                         Q.E.D.
+--     Step: 1.2.4                         Q.E.D.
 --     Step: 1.Completeness                Q.E.D.
 --   Result:                               Q.E.D.
 -- [Proven] gcdMaximal :: Ɐa ∷ Integer → Ɐb ∷ Integer → Ɐx ∷ Integer → Bool
 gcdMaximal :: TP (Proof (Forall "a" Integer -> Forall "b" Integer -> Forall "x" Integer -> SBool))
 gcdMaximal = do
 
-   dAbs  <- recall "dvdAbs" dvdAbs
+   dAbs <- recall "dvdAbs" dvdAbs
+   comm <- recall "gcdComm" commutative
 
    eDiv <- lemma "eDiv"
                  (\(Forall @"x" x) (Forall @"y" y) -> y ./= 0 .=> x .== (x `sEDiv` y) * y + x `sEMod` y)
@@ -501,7 +507,7 @@ gcdMaximal = do
    mNGCD <- sInduct "mNGCD"
                     (\(Forall @"a" a) (Forall @"b" b) (Forall @"x" x) ->
                           a .>= 0 .&& b .>= 0 .&& x `dvd` a .&& x `dvd` b .=> x `dvd` nGCD a b)
-                    (\_a b _x -> b) $
+                    (\_a b _x -> b, []) $
                     \ih a b x -> let g = nGCD a b
                               in [a .>= 0, b .>= 0, x `dvd` a .&& x `dvd` b]
                               |- x `dvd` g
@@ -524,7 +530,10 @@ gcdMaximal = do
                                            ?? dAbs     `at` (Inst @"a" x, Inst @"b" b)
                                            =: sTrue
                                            =: qed
-                        , abs a .<  abs b ==> x `dvd` nGCD (abs b) (abs a)
+                        , abs a .<  abs b ==> x `dvd` gcd a b
+                                           ?? comm `at` (Inst @"a" a, Inst @"b" b)
+                                           =: x `dvd` gcd b a
+                                           =: x `dvd` nGCD (abs b) (abs a)
                                            ?? mNGCD    `at` (Inst @"a" (abs b), Inst @"b" (abs a), Inst @"x" x)
                                            ?? dAbs     `at` (Inst @"a" x, Inst @"b" a)
                                            ?? dAbs     `at` (Inst @"a" x, Inst @"b" b)
@@ -694,7 +703,7 @@ gcdEvenEven = do
 
    nGCDEvenEven <- sInduct "nGCDEvenEven"
                            (\(Forall @"a" a) (Forall @"b" b) -> a .>= 0 .&& b .>= 0 .=> nGCD (2*a) (2*b) .== 2 * nGCD a b)
-                           (\_a b -> b) $
+                           (\_a b -> b, []) $
                            \ih a b -> [a .>= 0, b .>= 0]
                                    |- nGCD (2*a) (2*b)
                                    =: cases [ b .== 0 ==> trivial
@@ -841,7 +850,7 @@ gcdSubEquiv = do
    -- First prove over the non-negative numbers:
    nEq <- sInduct "nGCDSubEquiv"
                   (\(Forall @"a" a) (Forall @"b" b) -> a .>= 0 .&& b .>= 0 .=> nGCDSub a b .== nGCD a b)
-                  (\a b -> a + b) $
+                  (\a b -> a + b, []) $
                   \ih a b -> [a .>= 0, b .>= 0]
                           |- nGCDSub a b
                           =: cases [ a .== b             ==> nGCD a b =: qed
@@ -942,7 +951,7 @@ gcdBinEquiv = do
    -- First prove over the non-negative numbers:
    nEq <- sInduct "nGCDBinEquiv"
                   (\(Forall @"a" a) (Forall @"b" b) -> a .>= 0 .&& b .>= 0 .=> nGCDBin a b .== nGCD a b)
-                  (\a b -> tuple (a, b)) $
+                  (\a b -> tuple (a, b), []) $
                   \ih a b -> [a .>= 0, b .>= 0]
                           |- nGCDBin a b
                           =: cases [ a .== 0               ==> trivial
