@@ -9,14 +9,16 @@
 -- Main entry point to the test suite
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE ScopedTypeVariables #-}
-
 {-# OPTIONS_GHC -Wall -Werror #-}
 
 module Main(main) where
 
 import Test.Tasty
 
+import qualified TestSuite.ADT.ADT
+import qualified TestSuite.ADT.Expr
+import qualified TestSuite.ADT.MutRec
+import qualified TestSuite.ADT.PExpr
 import qualified TestSuite.Arrays.InitVals
 import qualified TestSuite.Arrays.Memory
 import qualified TestSuite.Arrays.Query
@@ -120,6 +122,7 @@ import qualified TestSuite.Queries.UISat
 import qualified TestSuite.Queries.UISatEx
 import qualified TestSuite.Queries.Uninterpreted
 import qualified TestSuite.QuickCheck.QC
+import qualified TestSuite.CompileTests.SCase
 import qualified TestSuite.Transformers.SymbolicEval
 import qualified TestSuite.Uninterpreted.AUF
 import qualified TestSuite.Uninterpreted.Axioms
@@ -128,8 +131,13 @@ import qualified TestSuite.Uninterpreted.Sort
 import qualified TestSuite.Uninterpreted.Uninterpreted
 
 main :: IO ()
-main = defaultMain $ testGroup "SBV" [
-                        TestSuite.Arrays.InitVals.tests
+main = do sCaseTests <- TestSuite.CompileTests.SCase.tests
+          defaultMain $ testGroup "SBV" [
+                        TestSuite.ADT.ADT.tests
+                      , TestSuite.ADT.Expr.tests
+                      , TestSuite.ADT.MutRec.tests
+                      , TestSuite.ADT.PExpr.tests
+                      , TestSuite.Arrays.InitVals.tests
                       , TestSuite.Arrays.Memory.tests
                       , TestSuite.Arrays.Query.tests
                       , TestSuite.Arrays.Caching.tests
@@ -240,4 +248,5 @@ main = defaultMain $ testGroup "SBV" [
                       , TestSuite.Uninterpreted.Function.tests
                       , TestSuite.Uninterpreted.Sort.tests
                       , TestSuite.Uninterpreted.Uninterpreted.tests
+                      , sCaseTests
                       ]
