@@ -13,14 +13,11 @@
 
 {-# LANGUAGE CPP                 #-}
 {-# LANGUAGE DataKinds           #-}
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE FlexibleInstances   #-}
-{-# LANGUAGE Rank2Types          #-}
 {-# LANGUAGE QuasiQuotes         #-}
+{-# LANGUAGE Rank2Types          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
 {-# LANGUAGE TypeApplications    #-}
 
@@ -41,8 +38,8 @@ import qualified Data.Char       as C
 import qualified Data.SBV.Char   as SC
 import qualified Data.SBV.List   as SL
 
-data Day = Mon | Tue | Wed | Thu | Fri | Sat | Sun deriving (Bounded, Enum, Eq)
-mkSymbolicEnumeration  ''Day
+data Day = Mon | Tue | Wed | Thu | Fri | Sat | Sun deriving (Show, Bounded, Enum, Eq)
+mkSymbolic  [''Day]
 
 -- Test suite
 tests :: TestTree
@@ -1029,8 +1026,6 @@ genEnums =
 
         -- NB. Precision here is important. If you pick too small of a significand
         -- size then you can turn this enumeration into an infinite list, busting the tests.
-        -- For details see: https://gitlab.haskell.org/ghc/ghc/-/issues/15081
-        -- And likewise, constant folding doesn't happen, too difficult for z3
         fps :: [FloatingPoint 5 8]
         -- fps = [-3.4, -3.2 .. 3.5]
         fps = []
@@ -1046,6 +1041,8 @@ genEnums =
 
 -- Quiet GHC about unused enum elts
 _unused :: SDay
-_unused = undefined sMon sTue sWed sThu sFri sSat sSun
+_unused = undefined  sMon  sTue  sWed  sThu  sFri  sSat  sSun
+                    isMon isTue isWed isThu isFri isSat isSun
+                    (sCaseDay @SInteger)
 
 {- HLint ignore module "Reduce duplication" -}
