@@ -10,14 +10,10 @@
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE DataKinds           #-}
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE OverloadedLists     #-}
 {-# LANGUAGE QuasiQuotes         #-}
-{-# LANGUAGE RankNTypes          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
 {-# LANGUAGE TypeApplications    #-}
 
@@ -46,7 +42,7 @@ import Data.Proxy
 import Utils.SBVTestFramework
 
 data P
-mkUninterpretedSort ''P
+mkSymbolic [''P]
 
 drinker :: Predicate
 drinker = pure $ quantifiedBool $ \(Exists x) (Forall y) -> d x .=> d y
