@@ -9,14 +9,12 @@
 -- Testing fresh-vars in query mode
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE OverloadedLists     #-}
 {-# LANGUAGE OverloadedStrings   #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TypeApplications    #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -26,11 +24,8 @@ import Data.SBV.Control
 
 import Utils.SBVTestFramework
 
-data BinOp  = Plus | Minus | Times deriving (Enum, Bounded)
-mkSymbolicEnumeration ''BinOp
-
-_unused :: a
-_unused = error "stop GHC from complaining unused names" sPlus sMinus sTimes
+data BinOp  = Plus | Minus | Times
+mkSymbolic [''BinOp]
 
 -- Test suite
 tests :: TestTree
